{***************************************************************************}
{ VISOR RESIDENTE

  Copyright (c) 1999 Marco A. Alvarado

  Muestra el contenido de los registros del procesador y la memoria cuando
  se llama su interrupcin. Esta herramienta es til para revisar programas
  residentes. Es similar a un debugger.

  ADVERTENCIA:

    VISORE es capaz de interferir con el funcionamiento del sistema
    operativo. Puede daar la informacin que contiene su computadora, y en
    el peor de los casos, los dispositivos electrnicos. Su equipo estar
    seguro mientras no ejecute este programa. Antes de correrlo estudie muy
    bien el cdigo fuente, hasta que comprenda la tarea de cada rutina.

    Yo constru esta herramienta para mi propio uso. La he utilizado decenas
    de veces en mi computadora y hasta la fecha no he tenido problemas.

  Respuestas:

    marcalva@sol.racsa.co.cr

  Empleo:

    1. Coloque las llamadas a la interrupcin de VISORE en su programa. Por
       ejemplo agregue la siguiente lnea:

         ASM INT $80 END;

    2. Ejecute VISORE slo una vez. Como es un programa residente se cargar
       en memoria y se mantendr ah. Presione [Enter] para continuar.
    3. Ejecute su programa. }
{***************************************************************************}

{$M 4096, 0, 0}
PROGRAM VISOR_RESIDENTE;

  USES
    DOS,
    CRT;

  TYPE
    S_WORD = STRING [4];
    S_NOMBRE = STRING [12];

  CONST
    INTERRUPCION = $80;  {Este es el nmero de interrupcin que usa VISORE.
      CUIDADO: Cambielo por una interrupcin libre, si es necesaio.}
    DIGITOS : ARRAY [0..$F] OF CHAR = '0123456789ABCDEF';
    COLOR = 7;
    COLOR_REGISTRO = 3;
    COLOR_REGISTRO_CAMBIADO = 11;
    COLOR_CADENA = 14;
    COLOR_MENU = 15;
    VIEJO_FLAGS : WORD = 0;
    VIEJO_CS : WORD = 0;
    VIEJO_IP : WORD = 0;
    VIEJO_AX : WORD = 0;
    VIEJO_BX : WORD = 0;
    VIEJO_CX : WORD = 0;
    VIEJO_DX : WORD = 0;
    VIEJO_SI : WORD = 0;
    VIEJO_DI : WORD = 0;
    VIEJO_DS : WORD = 0;
    VIEJO_ES : WORD = 0;
    VIEJO_BP : WORD = 0;
    VIEJO_SP : WORD = 0;
    VIEJO_SS : WORD = 0;
    MAS : WORD = 0;
    FACTOR : BYTE = 1;
    CONTENIDO : BYTE = 1;
    CONTENIDO_AUTOMATICO : BOOLEAN = TRUE;
    PRIMERA_LLAMADA : BOOLEAN = TRUE;
    CORRECCION : WORD = 0;
    CORRECCION_ACTIVA : BOOLEAN = TRUE;
    ERROR : INTEGER = 0;

  PROCEDURE MUESTRA_BYTE_HEXADECIMAL (X : BYTE);
  BEGIN
    WRITE (DIGITOS [X SHR 4], DIGITOS [X AND $F]);
  END;

  PROCEDURE MUESTRA_WORD_HEXADECIMAL (X : WORD);
  BEGIN
    WRITE (DIGITOS [HI (X) SHR 4], DIGITOS [HI (X) AND $F],
      DIGITOS [LO (X) SHR 4], DIGITOS [LO (X) AND $F]);
  END;

  FUNCTION BYTE_ASCII (X : BYTE) : CHAR;
  BEGIN
    IF X < 32 THEN BYTE_ASCII := '' ELSE BYTE_ASCII := CHAR (X);
  END;

  PROCEDURE MUESTRA_REGISTRO (NOMBRE : S_NOMBRE; VIEJO, ACTUAL : WORD);
  BEGIN
    IF VIEJO = ACTUAL THEN TEXTCOLOR (COLOR_REGISTRO) ELSE
      TEXTCOLOR (COLOR_REGISTRO_CAMBIADO);
    WRITE (NOMBRE, '=');
    MUESTRA_WORD_HEXADECIMAL (ACTUAL);
    WRITE ('  ');
  END;

  PROCEDURE MUESTRA_CADENA (NOMBRE : S_NOMBRE; SEGMENTO, CAMPO : WORD);

    VAR
      INDICE : BYTE;

  BEGIN
    TEXTCOLOR (COLOR_CADENA);
    WRITE (NOMBRE, '=');

    FOR INDICE := 1 TO 16 DO
    BEGIN
      MUESTRA_BYTE_HEXADECIMAL (BYTE (PTR (SEGMENTO, CAMPO)^));
      WRITE (' ');
      INC (CAMPO);
    END;

    WRITE (' ');
    DEC (CAMPO, 16);

    FOR INDICE := 1 TO 16 DO
    BEGIN
      WRITE (BYTE_ASCII (BYTE (PTR (SEGMENTO, CAMPO)^)));
      INC (CAMPO);
    END;

    WRITE (' ');
  END;

  PROCEDURE MUESTRA_BANDERAS (VIEJO, ACTUAL : WORD);
  BEGIN

    IF ACTUAL > 0 THEN
    BEGIN
      IF VIEJO = ACTUAL THEN TEXTCOLOR (COLOR_REGISTRO) ELSE
        TEXTCOLOR (COLOR_REGISTRO_CAMBIADO);
      IF ACTUAL AND 1 > 0 THEN WRITE ('cf ');
      IF ACTUAL AND 4 > 0 THEN WRITE ('pf ');
      IF ACTUAL AND 16 > 0 THEN WRITE ('af ');
      IF ACTUAL AND 64 > 0 THEN WRITE ('zf ');
      IF ACTUAL AND 128 > 0 THEN WRITE ('sf ');
      IF ACTUAL AND 256 > 0 THEN WRITE ('tf ');
      IF ACTUAL AND 512 > 0 THEN WRITE ('if ');
      IF ACTUAL AND 1024 > 0 THEN WRITE ('df ');
      IF ACTUAL AND 2048 > 0 THEN WRITE ('of ');
      WRITE (' ');
    END;

  END;

  PROCEDURE MUESTRA (FLAGS, CS, IP, AX, BX, CX, DX, SI, DI, DS, ES, BP :
    WORD); INTERRUPT;

    VAR
      NUEVO_SP,
      NUEVO_SS : WORD;
      TECLA : CHAR;
      X,
      Y : BYTE;

    PROCEDURE CAMBIA_REGISTROS;
    BEGIN
      VIEJO_FLAGS := FLAGS + 1;
      VIEJO_CS := CS + 1;
      VIEJO_IP := IP + 1;
      VIEJO_AX := AX + 1;
      VIEJO_BX := BX + 1;
      VIEJO_CX := CX + 1;
      VIEJO_DX := DX + 1;
      VIEJO_SI := SI + 1;
      VIEJO_DI := DI + 1;
      VIEJO_DS := DS + 1;
      VIEJO_ES := ES + 1;
      VIEJO_BP := BP + 1;
      VIEJO_SP := NUEVO_SP + 1;
      VIEJO_SS := NUEVO_SS + 1;
    END;

    PROCEDURE MUESTRA_CODIGO;

      VAR
        RELATIVO : WORD;

    BEGIN
      IF CORRECCION_ACTIVA THEN
      BEGIN
        RELATIVO := IP + CORRECCION + MAS;
        MUESTRA_CADENA ('cs:[' + DIGITOS [HI (RELATIVO) SHR 4] +
          DIGITOS [HI (RELATIVO) AND $F] +
          DIGITOS [LO (RELATIVO) SHR 4] +
          DIGITOS [LO (RELATIVO) AND $F] + ']    ',
          CS, IP + MAS);
      END ELSE
        MUESTRA_CADENA ('cs:[ip+' + DIGITOS [HI (MAS) SHR 4] +
          DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
          DIGITOS [LO (MAS) AND $F] + ']', CS, IP + MAS)
    END;

    PROCEDURE MUESTRA_REGISTROS;
    BEGIN
      WRITELN;
      MUESTRA_REGISTRO ('ax', VIEJO_AX, AX);
      MUESTRA_REGISTRO ('dx', VIEJO_DX, DX);
      MUESTRA_REGISTRO ('bx', VIEJO_BX, BX);
      MUESTRA_REGISTRO ('cx', VIEJO_CX, CX);
      MUESTRA_REGISTRO ('f', VIEJO_FLAGS, FLAGS);
      MUESTRA_BANDERAS (VIEJO_FLAGS, FLAGS);
      WRITELN;
      MUESTRA_REGISTRO ('cs', VIEJO_CS, CS);
      MUESTRA_REGISTRO ('ds', VIEJO_DS, DS);
      MUESTRA_REGISTRO ('es', VIEJO_ES, ES);
      MUESTRA_REGISTRO ('ss', VIEJO_SS, NUEVO_SS);
      WRITELN;
      MUESTRA_REGISTRO ('ip', VIEJO_IP, IP);
      MUESTRA_REGISTRO ('si', VIEJO_SI, SI);
      MUESTRA_REGISTRO ('di', VIEJO_DI, DI);
      MUESTRA_REGISTRO ('sp', VIEJO_SP, NUEVO_SP);
      MUESTRA_REGISTRO ('bp', VIEJO_BP, BP);
    END;

    PROCEDURE MUESTRA_CONTENIDO;
    BEGIN
      WRITELN;

      IF CONTENIDO_AUTOMATICO THEN
      BEGIN
        IF (VIEJO_SP <> NUEVO_SP) OR (VIEJO_BP <> BP) OR
          (VIEJO_SS <> NUEVO_SS) THEN CONTENIDO := 1 ELSE
        IF (VIEJO_SI <> SI) OR (VIEJO_DI <> DI) OR
          ((VIEJO_DS <> DS) AND (CONTENIDO = 1)) OR
          ((VIEJO_ES <> ES) AND (CONTENIDO = 1)) THEN CONTENIDO := 2 ELSE
        IF (VIEJO_DX <> DX) OR (VIEJO_BX <> BX) OR
          ((VIEJO_DS <> DS) AND (CONTENIDO = 1)) OR
          ((VIEJO_ES <> ES) AND (CONTENIDO = 1)) THEN CONTENIDO := 3;
      END;

      CASE CONTENIDO OF

        1 : BEGIN
          MUESTRA_CADENA ('ss:[sp+' + DIGITOS [HI (MAS) SHR 4] +
            DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
            DIGITOS [LO (MAS) AND $F] + ']', NUEVO_SS,
            NUEVO_SP + MAS);
          WRITELN;
          MUESTRA_CADENA ('ss:[bp+' + DIGITOS [HI (MAS) SHR 4] +
            DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
            DIGITOS [LO (MAS) AND $F] + ']', NUEVO_SS,
            BP + MAS);
        END;

        2 : BEGIN
          MUESTRA_CADENA ('ds:[si+' + DIGITOS [HI (MAS) SHR 4] +
            DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
            DIGITOS [LO (MAS) AND $F] + ']', DS,
            SI + MAS);
          WRITELN;
          MUESTRA_CADENA ('es:[di+' + DIGITOS [HI (MAS) SHR 4] +
            DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
            DIGITOS [LO (MAS) AND $F] + ']', ES,
            DI + MAS);
        END;

        3 : BEGIN
          MUESTRA_CADENA ('ds:[dx+' + DIGITOS [HI (MAS) SHR 4] +
            DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
            DIGITOS [LO (MAS) AND $F] + ']', DS,
            DX + MAS);
          WRITELN;
          MUESTRA_CADENA ('es:[bx+' + DIGITOS [HI (MAS) SHR 4] +
            DIGITOS [HI (MAS) AND $F] + DIGITOS [LO (MAS) SHR 4] +
            DIGITOS [LO (MAS) AND $F] + ']', ES,
            BX + MAS);
        END;

      END;

    END;

    PROCEDURE MUESTRA_MENU;
    BEGIN
      TEXTCOLOR (COLOR_MENU);
      WRITELN;
      WRITE ('[c] ');
      IF CORRECCION_ACTIVA THEN WRITE ('c') ELSE WRITE (' ');
      WRITE (' [i] [0 - + *] ' + DIGITOS [FACTOR SHR 4] +
        DIGITOS [FACTOR AND $F] + ' [a p o f] ');
      IF CONTENIDO_AUTOMATICO THEN WRITE ('a') ELSE WRITE (' ');
      WRITE (' [intro] ');
    END;

    PROCEDURE MUESTRA_RESUMEN;
    BEGIN
      IF CORRECCION_ACTIVA THEN
      MUESTRA_REGISTRO ('i+', VIEJO_IP + CORRECCION + MAS,
        IP + CORRECCION + MAS)
      ELSE
      MUESTRA_REGISTRO ('ip', VIEJO_IP, IP);
      IF VIEJO_AX <> AX THEN MUESTRA_REGISTRO ('ax', VIEJO_AX, AX);
      IF VIEJO_BX <> BX THEN MUESTRA_REGISTRO ('bx', VIEJO_BX, BX);
      IF VIEJO_CX <> CX THEN MUESTRA_REGISTRO ('cx', VIEJO_CX, CX);
      IF VIEJO_DX <> DX THEN MUESTRA_REGISTRO ('dx', VIEJO_DX, DX);
      IF VIEJO_SI <> SI THEN MUESTRA_REGISTRO ('si', VIEJO_SI, SI);
      IF VIEJO_DI <> DI THEN MUESTRA_REGISTRO ('di', VIEJO_DI, DI);
      IF VIEJO_SP <> NUEVO_SP THEN
        MUESTRA_REGISTRO ('sp', VIEJO_SP, NUEVO_SP);
      IF VIEJO_BP <> BP THEN MUESTRA_REGISTRO ('bp', VIEJO_BP, BP);
      IF VIEJO_CS <> CS THEN MUESTRA_REGISTRO ('cs', VIEJO_CS, CS);
      IF VIEJO_DS <> DS THEN MUESTRA_REGISTRO ('ds', VIEJO_DS, DS);
      IF VIEJO_ES <> ES THEN MUESTRA_REGISTRO ('es', VIEJO_ES, ES);
      IF VIEJO_SS <> NUEVO_SS THEN
        MUESTRA_REGISTRO ('ss', VIEJO_SS, NUEVO_SS);
      MUESTRA_BANDERAS (VIEJO_FLAGS, FLAGS);
    END;

    PROCEDURE IGUALA_REGISTROS;
    BEGIN
      VIEJO_FLAGS := FLAGS;
      VIEJO_CS := CS;
      VIEJO_IP := IP;
      VIEJO_AX := AX;
      VIEJO_BX := BX;
      VIEJO_CX := CX;
      VIEJO_DX := DX;
      VIEJO_SI := SI;
      VIEJO_DI := DI;
      VIEJO_DS := DS;
      VIEJO_ES := ES;
      VIEJO_BP := BP;
      VIEJO_SP := NUEVO_SP;
      VIEJO_SS := NUEVO_SS;
    END;

  BEGIN

    ASM
      MOV NUEVO_SP, SP
      MOV NUEVO_SS, SS
    END;

    IF PRIMERA_LLAMADA THEN
    BEGIN
      CAMBIA_REGISTROS;
      PRIMERA_LLAMADA := FALSE;
    END;

    MUESTRA_CODIGO;
    MUESTRA_REGISTROS;
    MUESTRA_CONTENIDO;
    MUESTRA_MENU;

    REPEAT
      TECLA := READKEY;

      CASE TECLA OF

        'C', 'c' : BEGIN
          CORRECCION_ACTIVA := NOT (CORRECCION_ACTIVA);
          X := WHEREX;
          Y := WHEREY;
          GOTOXY (1, WHEREY - 6);
          MUESTRA_CODIGO;
          GOTOXY (1, WHEREY + 5);
          MUESTRA_MENU;
        END;

        'I', 'i' : BEGIN
          CAMBIA_REGISTROS;
          X := WHEREX;
          Y := WHEREY;
          GOTOXY (1, WHEREY - 6);
          MUESTRA_REGISTROS;
          GOTOXY (X, Y);
        END;

        '0' : BEGIN
          MAS := 0;
          X := WHEREX;
          Y := WHEREY;
          GOTOXY (1, WHEREY - 6);
          MUESTRA_CODIGO;
          GOTOXY (1, WHEREY + 3);
          MUESTRA_CONTENIDO;
          GOTOXY (X, Y);
        END;

        '-' : BEGIN
          DEC (MAS, FACTOR);
          X := WHEREX;
          Y := WHEREY;
          GOTOXY (1, WHEREY - 6);
          MUESTRA_CODIGO;
          GOTOXY (1, WHEREY + 3);
          MUESTRA_CONTENIDO;
          GOTOXY (X, Y);
        END;

        '+' : BEGIN
          INC (MAS, FACTOR);
          X := WHEREX;
          Y := WHEREY;
          GOTOXY (1, WHEREY - 6);
          MUESTRA_CODIGO;
          GOTOXY (1, WHEREY + 3);
          MUESTRA_CONTENIDO;
          GOTOXY (X, Y);
        END;

        '*' : BEGIN

          CASE FACTOR OF
            1 : FACTOR := 2;
            2 : FACTOR := 4;
            4 : FACTOR := 8;
            8 : FACTOR := 16;
            16 : FACTOR := 1;
          END;

          X := WHEREX;
          Y := WHEREY;
          GOTOXY (1, WHEREY - 1);
          MUESTRA_MENU;
          GOTOXY (X, Y);
        END;

        'A', 'a' : BEGIN
          CONTENIDO_AUTOMATICO := NOT (CONTENIDO_AUTOMATICO);
          GOTOXY (1, WHEREY - 3);
          MUESTRA_CONTENIDO;
          MUESTRA_MENU;
        END;

        'P', 'p' : BEGIN
          CONTENIDO := 1;
          CONTENIDO_AUTOMATICO := FALSE;
          GOTOXY (1, WHEREY - 3);
          MUESTRA_CONTENIDO;
          MUESTRA_MENU;
        END;

        'O', 'o' : BEGIN
          CONTENIDO := 2;
          CONTENIDO_AUTOMATICO := FALSE;
          GOTOXY (1, WHEREY - 3);
          MUESTRA_CONTENIDO;
          MUESTRA_MENU;
        END;

        'F', 'f' : BEGIN
          CONTENIDO := 3;
          CONTENIDO_AUTOMATICO := FALSE;
          GOTOXY (1, WHEREY - 3);
          MUESTRA_CONTENIDO;
          MUESTRA_MENU;
        END;

      END;

    UNTIL TECLA = #13;

    GOTOXY (1, WHEREY - 6);
    TEXTCOLOR (COLOR);
    WRITELN ('                                                                               ');
    WRITELN ('                                                                               ');
    WRITELN ('                                                                               ');
    WRITELN ('                                                                               ');
    WRITELN ('                                                                               ');
    WRITELN ('                                                                               ');
    WRITELN ('                                                                               ');
    GOTOXY (1, WHEREY - 7);
    MUESTRA_RESUMEN;
    TEXTCOLOR (COLOR);
    WRITELN;
    IGUALA_REGISTROS;
  END;

BEGIN

  IF PARAMSTR (1) <> '' THEN
  BEGIN
    VAL (PARAMSTR (1), CORRECCION, ERROR);

    IF ERROR <> 0 THEN
    BEGIN
      WRITELN ('El valor decimal de correccin no es vlido.');
      HALT (1);
    END;

  END;

  CLRSCR;
  TEXTCOLOR (COLOR);
  WRITE ('Instala el VISOR RESIDENTE en la interrupcin ');
  MUESTRA_BYTE_HEXADECIMAL (INTERRUPCION);
  WRITELN ('H.');
  WRITE ('Corrige la posicin de las instrucciones en ');
  MUESTRA_WORD_HEXADECIMAL (CORRECCION);
  WRITELN ('H.');
  SETINTVEC (INTERRUPCION, @MUESTRA);
  WRITELN;

  ASM
    INT INTERRUPCION
  END;

  KEEP (0);
END.